#include "Seeed_vl53l0x.h"

// Nano Pinout
//Optional interrupt and shutdown pins.
#define SHUTDOWN_PIN1 		5
#define SHUTDOWN_PIN2 		6
//Address selector.
#define ADDR_1				    A0
#define ADDR_2				    A1
#define ADDR_4				    A2
#define ADDR_8				    A3
#define EN485				      4
#define LED_PIN				    7


/////////////////////////
/// General define //////
/////////////////////////
#define SERIAL_USB_BAUD 115200
#define SERIAL_RS485_BAUD 9600
#define INTERVAL  300
#define LED_TIME  1000
#define RS485_ADD  3
#define RS485_BROAD 32
#define MAX_485_LEN 4
#define RS485_ADD_OFFSET  40


char A_8=0, A_4=0, A_2=0, A_1=0, addr_485 = 0, addr_broadcast = RS485_BROAD;
int level_1 = 0;
int level_2 = 0;
int rs485_enabled = 0;
const long interval = INTERVAL;           // interval for timer tick (milliseconds)
unsigned long previousMillis = 0;        // will store last time was updated
char led_status = 0;
// RS485 Message
char msg485buff[MAX_485_LEN];
char index_485= 0;
char  rs485_buffer[20];
int sens_1_enabled = 1;
int sens_2_enabled = 1;

Seeed_vl53l0x distanceSensor1;
Seeed_vl53l0x distanceSensor2;
VL53L0X_RangingMeasurementData_t measure;
VL53L0X_Error Status = VL53L0X_ERROR_NONE;

/////////////////////
// Check LEVEL
/////////////////////
void read_level(){
  level_1 = 0;
  level_2 = 0;
  if (sens_1_enabled > 0)
  {
      digitalWrite(SHUTDOWN_PIN1, HIGH);
      delay(20);
      Status = VL53L0X_ERROR_NONE;
      Status = distanceSensor1.VL53L0X_common_init();
      Status = distanceSensor1.VL53L0X_long_distance_ranging_init();
      delay(20);
      memset(&measure, 0, sizeof(VL53L0X_RangingMeasurementData_t));
      Status = distanceSensor1.PerformSingleRangingMeasurement(&measure);
      if (VL53L0X_ERROR_NONE == Status) {
        if (measure.RangeMilliMeter < 2000)
        {
          level_1 = (measure.RangeMilliMeter/10);
        }
        else
        {
          level_1 = 255;
        }
      }
      else
      {
        if (rs485_enabled == 0)
        {
          Serial.print("mesurement 1 failed !! Status code =");
          Serial.println(Status); 
        }
      }
      digitalWrite(SHUTDOWN_PIN1, LOW);

  }

  if (sens_2_enabled > 0)
  {
      digitalWrite(SHUTDOWN_PIN2, HIGH);
      delay(20);
      Status = VL53L0X_ERROR_NONE;
      Status = distanceSensor2.VL53L0X_common_init();
      Status = distanceSensor2.VL53L0X_long_distance_ranging_init();
      delay(3);
      memset(&measure, 0, sizeof(VL53L0X_RangingMeasurementData_t));
      Status = distanceSensor2.PerformSingleRangingMeasurement(&measure);
      if (VL53L0X_ERROR_NONE == Status)  {
        if (measure.RangeMilliMeter < 2000)
        {
          level_2 = (measure.RangeMilliMeter/10);
        }
        else
        {
          level_2 = 255;
        }
        
      }
      else
      {
        if (rs485_enabled == 0)
        {
          Serial.print("mesurement 2 failed !! Status code =");
          Serial.println(Status); 
        }
      }
      digitalWrite(SHUTDOWN_PIN2, LOW);//
  }
 if (rs485_enabled == 0)
  {
  if (sens_1_enabled > 0)
  {
    Serial.print("Distance 1 (cm): ");
    Serial.print(level_1);   
    Serial.println();
  }
  else 
  {
	Serial.println("Sensors 1 error");
  }
 
  if (sens_2_enabled > 0)
  {
    Serial.print("Distance 2 (cm): ");
    Serial.print(level_2);
    Serial.println();
  }
  else 
  {
	Serial.println("Sensors 2 error");
  }
  }  
}

void setup(void)
{
  //Pin init for shutdown TOF sensor
  pinMode(SHUTDOWN_PIN1 , OUTPUT);
  pinMode(SHUTDOWN_PIN2 , OUTPUT);
  digitalWrite(SHUTDOWN_PIN1, LOW);
  digitalWrite(SHUTDOWN_PIN2, LOW);
  //Pin init for LED
  pinMode(LED_PIN , OUTPUT);
  digitalWrite(LED_PIN, HIGH);
  //Pin RS485 EN
  pinMode(EN485, OUTPUT);
  //Pin init and read physical address
  pinMode(ADDR_8, INPUT_PULLUP);    // sets the digital pin as input
  A_8 = digitalRead(ADDR_8);
  pinMode(ADDR_4, INPUT_PULLUP); 	  // read the input pin
  A_4 = digitalRead(ADDR_4);
  pinMode(ADDR_2, INPUT_PULLUP);  	// read the input pin
  A_2 = digitalRead(ADDR_2);
  pinMode(ADDR_1, INPUT_PULLUP);  	// read the input pin
  A_1 = digitalRead(ADDR_1);
  addr_485 = A_1 + A_2 * 2 + A_4 * 4 + A_8 * 8 + RS485_ADD_OFFSET;

	if (addr_485>0) 
	{
	  rs485_enabled = 1;
	  digitalWrite(EN485, LOW);
	}
	else 
	{
	  rs485_enabled = 0;
      digitalWrite(EN485, HIGH);
	}
	
	delay(100);
	if (rs485_enabled == 0)
	{
	  Serial.begin(9600);
	  Serial.println("Level sensor V1.2");
	}
	else
	{
	  Serial.begin(9600);
	}
	read_level();
	for (index_485 = 0; index_485< MAX_485_LEN; index_485++) msg485buff[index_485]=0; 
	if ((rs485_enabled == 0) && (sens_1_enabled > 0 ) && (sens_2_enabled > 0 ))
	{   
		Serial.println("Sensor 1 and 2 online!");
	} 
}

void loop(void)
{
  //digitalWrite(LED_PIN, HIGH); //LED ON
	char addr ='c';
	char centr_cmd ='c';
	char centr_payload ='c';
	unsigned long currentMillis = millis();
	//Pin init and read physical address
	A_8 = digitalRead(ADDR_8);
	A_4 = digitalRead(ADDR_4);
	A_2 = digitalRead(ADDR_2);
	A_1 = digitalRead(ADDR_1);
	addr_485 = A_1 + A_2 * 2 + A_4 * 4 + A_8 * 8 + RS485_ADD_OFFSET;

	rs485_buffer[0]= 'A';
	rs485_buffer[1]= addr_485;
	rs485_buffer[2]= 'L';
	rs485_buffer[3]= level_1;
	rs485_buffer[4]= 'V';
	rs485_buffer[5]= level_2;
	rs485_buffer[6]= '!';
	rs485_buffer[7]= 0;

	msg485buff[0] = 0;
	msg485buff[1] = 0;
	msg485buff[2] = 0;
	msg485buff[3] = 0;
	
	if (rs485_enabled >0)
	{  	
		if (Serial.available() > 0)
		{
			msg485buff[0] = Serial.read();
			delay(2);
			msg485buff[1] = Serial.read();
			delay(2);
			msg485buff[2] = Serial.read();
			delay(2);
			msg485buff[3] = Serial.read();
		}

	   addr = msg485buff[0];
	   centr_cmd = msg485buff[1];
	   centr_payload = msg485buff[2];
	   if(((addr== addr_485)) && (msg485buff[3] == '!'))
	   {
			read_level();
      // Prepare new buffer
      memset(msg485buff, 0, sizeof(msg485buff));
      rs485_buffer[0]= 'A';
      rs485_buffer[1]= addr_485;
      rs485_buffer[2]= 'L';
      rs485_buffer[3]= level_1;
      rs485_buffer[4]= 'V';
      rs485_buffer[5]= level_2;
      rs485_buffer[6]= '!';
      rs485_buffer[7]= 0; 
			if (addr== addr_485)
			{
				digitalWrite(EN485,HIGH);//DE/RE=HIGH Transmit Enabled 
				Serial.write(rs485_buffer, 8 );		
				delay(100);
				digitalWrite(EN485,LOW);//DE/RE=LOW Receive Enabled
			}
		}
		Serial.flush();	
		digitalWrite(EN485,LOW);//DE/RE=LOW Receive Enabled
	}
    // Main timeout
	if (currentMillis - previousMillis >= (interval)) 
	{
		previousMillis = currentMillis;
		led_status++;
		digitalWrite(LED_PIN, led_status%2); //LED TOGGLE
	}
}
